#pylint: disable=R0904,C0103,E1101,C0301,W0702

"""
It was easier to write code in Python than in Javascript, so whenever a user's
web browser needs to do something complicated, we have it open a websocket
connection to this daemon, and then the daemon will actually do all the work.

Right now, this daemon is required for both password generation and everyday
login. In practice, requiring a daemon for login is probably infeasible (eg,
now I can't log in on a friend's computer) so we would probably want to move
the SRP code to Javascript.
"""

from tornado import websocket, web, ioloop
import json
from client_generate import GenerateClient
from client_login import LoginClient
import os
import requests
import traceback

class GenerateSocketHandler(websocket.WebSocketHandler):
    """Listen for requests to generate a password"""

    def on_message(self, data):
        data = json.loads(data)
        username = data['username']

        # generate a password
        client = GenerateClient(username, data['generate_endpoint'])

        password = client.get_password()
        # copy it to the clipboard
        os.system("printf '%%s' '%s' | pbcopy" % password)

        # register the password with the authentication service to receive an
        # authentication token
        payload = dict(
            username = username,
            commitment_P = client.get_commitment_P(),
            public_key = client.get_public_key_dict(),
        )
        headers = {'content-type': 'application/json'}
        r = requests.post(data['register_endpoint'], data=json.dumps(payload), headers=headers)
        if r.status_code != requests.codes.ok:
            raise Exception

        # send the password and authentication token back to the browser
        ret_data = dict(
            password = password,
            authentication_token = r.text,
        )
        self.write_message(json.dumps(ret_data))

class LoginSocketHandler(websocket.WebSocketHandler):
    """Listen for requests to login"""

    def on_message(self, data):
        data = json.loads(data)
        try:
            # ask the authentication service for an authentication token
            client = LoginClient(data['username'], data['password'], data['login_endpoint'])
            authentication_token = client.get_token()
        except:
            # if login fails, still send something back to the browser
            authentication_token = None
            traceback.print_exc()

        # send the token back to the browser
        ret_data = dict(
            success = authentication_token is not None,
            authentication_token = authentication_token,
        )
        self.write_message(json.dumps(ret_data))

app = web.Application([
    (r'/generate', GenerateSocketHandler),
    (r'/login', LoginSocketHandler),
], debug=True)

if __name__ == '__main__':
    app.listen(8888)
    ioloop.IOLoop.instance().start()
