#pylint: disable=C0301,C0103,F0401,R0903

"""
This is the main service for our sample webserver (a bank). In the Zoobar
analogy, this would correspond to the static+dynamic service (before the two
were split into separate services).
"""

from flask import Flask, request, render_template, url_for, redirect, session
from functools import wraps
app = Flask(__name__)

import service_keys
from Crypto.PublicKey import RSA
MAIN_SVC_PRIVATE_KEY = RSA.importKey('-----BEGIN RSA PRIVATE KEY-----\nMIIJKQIBAAKCAgEAvq4JMs2L3tz6BKs8o70HENpWv0LprOe8+iRG4eSkWF2QKWmY\nGFLFYUjtstnN1QjkRYFBQknfCwmL69+/fPlzMuUW1lP/SZcm2poyueY045EbEX91\n7hWzuIqJm+FyEjmAdhb/iGBpyElEtkwaZdU/CXLBS1Bw+XJWfL+Z2Ptpb+GTv5I/\npfP+nMadtKMQxZCAOC7OQNUpOJSex2QCtrqUtiDqr/R7+GEtRhneAtpWQ2fUdiGD\nyh05hbjA07ar1z3cDrcqkJhFYq1uwaD/6EoNMibVUV48vT9iM4cvBBmhxoC9ZJIE\nlKjN4JR6WgtTMUAj0o3sXFGqsg/UswI5fBY9mIOu5pcBXXpMbTLgRYm3KCSFBBPA\nXVQx6tz0d52d4D4b04SYFnVQha7iipfq3NPLVKaQGG5xKqbX05ImFpOKMXYSGixd\nJegZtQN8snl0Qg/lE/PTql7A3DQcAZxnA/o0zPvWaywtTQN6sqVq5uyUv/Upajyi\n1UCnqhBdDowRhszr80b5G6Vt5ZyXuswswO+OAPy0AGvPB9y7LTJWhixx+1UkWs+I\nh31701kTgUj1SuEGxsudDFhvuvrVfX6YTyxGRzvZTUZmWJci+2J0pYVUptPQzuYT\n5ldh266p3RABorLraLNDVg0cN1SprHch/0LiwR9i0oGKxGm05+honCrJ2l8CAwEA\nAQKCAgAEie/+HZZuv6PT6J+Q5jb817Wo59bYHswIyERHrEGdONztn23XZmD8Z9gA\nod6ca5HDdcgv48JDaP7DppfQSb/MPlfE0ksJXWe3XGRIuIoZwKJ8UCRmfm3Avst3\nwm4MsQG1zeZXQLRD6DqSfE1Ej0bo2qpvOIL37RjR712iJJnl5+C0dq69LImrAa9P\nVXJYwY/LxpFySaCpkCAXbvIBl39itnXfqlopF2cUfBkkyD21bQtJpJKWhQuRe0xb\nvcVpYjeSwhlpR7afGrHAOizdQqX9w57gSPX5K0ETKOydlF6dcwMPGcqyHzKHXgFr\nk0lvqq8vm+E8cVU219mgSTVFFuJwbGI0uw456kMyiHoF+kypVG4fYtqH21ocetjs\nU/FxCkf/XnFesRtEjnCs5RDuayzz7xUCzFC2N2QMviOWFo5S6zKywEcnHtsy4hTf\nb2SRsAvnmR+wKRyJsDBU1xYVqhpXTcvSE9kFQHYZDCEyGHWv6ibq6jm/zhXzb4Id\nYIac2VeibsrknxPxDLdEAC++3yCRJsUyUVqZ68zl9WxJ2SZILuBRPsB6RFS5f/pK\nHGn804cx7X0Mo9YnCHVRN429VXGL1JUT/Wjl6DtDygAG/Oxn4lgnE+tGO1I5bUua\nfqGr5P3ReprqcUsrcsBSrr2dTAqSZTWPXCyCkR7LPqrAf5O3QQKCAQEAycAcJLC1\np1KI8Yy3HXgx8rNiFwLnir4K3GYJ5LupLoK7sZ2e8VjU+h7IpdKf1XDgqaEVAcrQ\ngW9/WirSwqpa4jA7kUYNAdQQYIOLIvw0Hm07mwIuX3zcGBAfVUSMQ+8Gr46vMI8T\nB08iz9DPVUYfB6GPDU/l10bYaPqyzOFDXpEi+981MPiY0bifo6iLfw7ZBcbk6mRB\nAKBzrxScuIAcPU1iA21r+zebD0NmLokmubcN/eePlp6meRhGW4LkxBJJDHI1CURj\nwOaQJbcvIQ6PgM2flALofHmhrQ7sPS+p/cGMvWuYUCPCc29aQjvZ0GuSo+yfXBZ8\n8eb1yad7bBH3uwKCAQEA8fPb9my4z4LzZ/aZh2HP8ZSd43tf0QGv7UpMGmX9/6yD\nuznz07pfB+04eFvu3PrDaG4MZYJ9k1AFFxtkSO4JhyUo8rycgh35GnrfvtZmdSUy\nGzucr7mx4KNYhvudwMYzPVOSBuzQt1qTaEUM0RteABoPgpQ+NTvPcfXnyQlPQPur\nyuIzln/IFvaeh7zQiYhM1RBgApQ/5sz6xdexbAvov8xzCtascV2E+RvoHbA0UKlp\nRAfYglfUaBTmQXMi/gxmNIFB1z5LTDKRwgWBZQeZf6RtqvPBsiKGOos4qXBohE7e\nhDgPwU0DAW3xNinAFpbyIOJqAA9mDT4hzGAdvLbDrQKCAQEAjGaNL1tqj074zmDG\n7lM/L09269yxlfQ7SrYYHH4cdqm0RR8UBZF0ZbwWlo/Fe5xaPLJFFEI4aFtXnUA/\n/AjhxpcaXTcKxkL2QrRIl8g73RDXckBKVVlnMbfGlmhl1MD7PiFl+b0X2PZRm54N\nK6Lo5T6hdvorUW2UHTqaQ8pgP+dxGSd0tOfMMebx8crRiVKj5UlM9XFfazg+SZFm\nn19IR8EafJghlWpVFoIpEimQMZ4atp/3ZTJzV8J09CpbWbf/7hFsp05YvV224whg\nW/juFQ0ECNUdRcK/qNfBv7Lz8gxIDWpbTSoVoI/VmFJeepTiMAHzxPpBaXb5haRw\nV/0JEQKCAQEA4s0dvuRxZQuykRzHE/HrU+BlFI1c5vX27K89oW1D6HvVTBbp6eoF\nFNAYpDJBOWHS9Lxb4HeusP1MmDl0Cy0fP1DlDo0hBJOGI31U6XXeuybvzjlWCh8+\nj31vCbEpUkA4Xd9NvjIVABrmnNJFTmqh9eR374Z4vbvgHEpPSNLyOM7xXZAmnYgQ\njv4E4uZqcNGUfBEKfsBrGjXBSN2Ekeyj8855dIqBQQ9SC5VOQy4eDBKR+1SDbsNX\n92zH3khIX+5NjCAyEMtzF+07Vq2EeqjiCZWfu6UdR5c3Tf+dUK9vOb8YtMY8do1z\nKM7fUk9WoZZDR/IVPRMyLW21jvf2SCTmTQKCAQBRemWE4BC+qY920j9gyifWXY1o\nC8jUZR8P0TWtnT24bFaAErC2U4gnXX0q7uzryR+tsX5k6FqrVWccCu8+f0Go1BPk\nMU6YGL+g5iYkxt5WbZQacs/S9VcAOOcGE/wB+wjMXoWgKn849xGGT0HgR3jghLgT\ng8a8Cc79FiNnWUSiD4pggdQQvhLM7WP1dXBh8IQ4sMGtuia+JKu4ulNMVryW+M3A\n5MDYG6leepzJyww02872zBwD33IzF0GHJJeHjGBuvVQo3e7r9SIKgGKQYA119s/y\nYGTheU0/msKvgZa1u5JKp8i3cufsMPl74tFmouCar1Bdg+74dgLFWDyNOte7\n-----END RSA PRIVATE KEY-----')

# because the client and server are running on the same computer (right now),
# the lib directory is located elsewhere, and we have to hack $PATH to find it
import sys
sys.path.append('../lib')
from pwgen_lib import verify_and_decrypt_dict

import time

class User(object):
    """A simple class representing a user"""

    def __init__(self, username, last_login):
        self.username = username
        self.last_login = last_login

# right now, store all data in-memory (because this is just a proof-of-concept)
users = {}

def requires_auth(f):
    """A wrapper to restrict certain routes to authenticated users only"""
    @wraps(f)
    def decorated(*args, **kwargs):
        """
        Check for username in the session dict, which is sufficient because
        Flask stores sessions in an encrypted cookie
        """
        if not session or not "username" in session:
            return redirect(url_for("index"))
        return f(*args, **kwargs)
    return decorated

@app.route('/')
def index():
    """Show the log-in page"""
    return render_template("index.html")

@app.route('/profile')
@requires_auth
def profile():
    """Show a sample logged-in page"""
    return render_template("profile.html")

@app.route('/register', methods=['POST'])
def register():
    """The log-in page submits to this endpoint once a first-time user has negotiated a password"""

    session.pop("username", None)

    f = request.form
    username = f.get("username")
    if username in users:
        # user is already registered
        return redirect(url_for("index"))

    token = verify_and_decrypt_dict(f.get("authentication_token"), MAIN_SVC_PRIVATE_KEY, service_keys.AUTH_SVC_PUBLIC_KEY)
    # make sure the token is a log-in token for the correct user
    if token['username'] != username or token['logged_in'] != True:
        return redirect(url_for("index"))
    # authentication tokens expire after 60 seconds
    if time.time() - token['timestamp'] > 60:
        return redirect(url_for("index"))

    # success! register the user and log them in
    users[username] = User(username, token['timestamp'])
    session["username"] = username
    return redirect(url_for("profile"))

@app.route('/login', methods=['POST'])
def login():
    """The log-in page submits to this endpoint once a user has finished SRP"""

    session.pop("username", None)

    f = request.form
    username = f.get("username")
    if username not in users:
        # user does not exist
        return redirect(url_for("index"))

    token = verify_and_decrypt_dict(f.get("authentication_token"), MAIN_SVC_PRIVATE_KEY, service_keys.AUTH_SVC_PUBLIC_KEY)
    # make sure the token is a log-in token for the correct user
    if token['username'] != username or token['logged_in'] != True:
        return redirect(url_for("index"))
    # authentication tokens expire after 60 seconds
    if time.time() - token['timestamp'] > 60:
        return redirect(url_for("index"))

    u = users[username]
    # prevent replay attacks by only accepting each token once
    if token['timestamp'] <= u.last_login+1:
        return redirect(url_for("index"))
    u.last_login = token['timestamp']

    # success! log the user in
    session["username"] = username
    return redirect(url_for("profile"))

@app.route('/logout')
def logout():
    """This endpoint logs a user out"""

    session.pop("username", None)
    return redirect(url_for("index"))

# Flask requires a secret key in order to make encrypted session cookies
app.secret_key = '.V\x1f\x11E\xa75m\x9e\x93\xac\xc3XB\x92\xfb4\x1f\xd8\x9f>\x990\x1f'

if __name__ == '__main__':
    app.run(port=8080, debug=True)
